"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const es_1 = require("agenda/es");
const inversify_1 = require("inversify");
class InversifyAgendaTasksConfiguration {
    constructor() {
        this.tasks = [];
        this.intervals = {};
    }
    register(target, key, options, interval, focus) {
        this.tasks.push({ key, target, options, focus });
        if (interval) {
            if (typeof interval === 'string' || typeof interval === 'number') {
                this.intervals[interval] = this.intervals[interval] || [];
                this.intervals[interval].push({ key });
            }
            else {
                this.intervals[interval.interval] = this.intervals[interval.interval] || [];
                this.intervals[interval.interval].push({
                    data: interval.data,
                    key
                });
            }
        }
    }
}
exports.InversifyAgendaTasksConfiguration = InversifyAgendaTasksConfiguration;
exports.inversifyAgendaTasksConfiguration = new InversifyAgendaTasksConfiguration();
function task(key, int, options, focus) {
    return (target) => {
        inversify_1.decorate(inversify_1.injectable(), target);
        if (int && Array.isArray(int)) {
            int.forEach((interval, idx) => exports.inversifyAgendaTasksConfiguration.register(target, `${key}.${idx}`, options, interval, focus));
        }
        else if (int && !Array.isArray(int)) {
            exports.inversifyAgendaTasksConfiguration.register(target, key, options, int, focus);
        }
        else {
            exports.inversifyAgendaTasksConfiguration.register(target, key, options, undefined, focus);
        }
    };
}
exports.task = task;
function ftask(key, int, options) {
    return task(key, int, options, true);
}
exports.ftask = ftask;
function xtask(key, int, options) {
    console.log('xtask() // ignoring', key);
}
exports.xtask = xtask;
class InversifyAgenda {
    constructor(container, config) {
        this.container = container;
        this.config = config;
        this.errorHandlers = [];
        if (this.config.agenda && this.config.db) {
            throw new Error('Cannot use agenda instance and db configuration at the same time');
        }
        if (!this.config.agenda && !this.config.db) {
            throw new Error('Yu have to configure agenda or db connection');
        }
        if (this.config.db) {
            this.config.agenda = new es_1.Agenda({
                db: {
                    address: this.config.db.address,
                    collection: this.config.db.collection,
                    options: this.config.db.options
                }
            });
        }
        this.container.bind(InversifyAgendaService).toConstantValue(new InversifyAgendaService(this.config.agenda));
    }
    onError(hanlder) {
        this.errorHandlers.push(hanlder);
        return this;
    }
    build() {
        let tasks = exports.inversifyAgendaTasksConfiguration.tasks;
        const focused = exports.inversifyAgendaTasksConfiguration.tasks.filter(t => t.focus);
        if (focused.length) {
            tasks = focused;
        }
        tasks.forEach(task => {
            if (!this.container.isBound(task.target)) {
                this.container.bind(task.target).toSelf();
            }
            this.defineTaskService(this.container, this.config.agenda, task.key, task.target, task.options);
        });
        this.config.agenda.on('ready', () => Object.keys(exports.inversifyAgendaTasksConfiguration.intervals)
            .forEach(interval => exports.inversifyAgendaTasksConfiguration.intervals[interval]
            .forEach(d => {
            console.log('Agenda config every', interval, d.key, d.data);
            this.config.agenda.every(interval, d.key, d.data);
        })));
        return this.config.agenda;
    }
    defineTaskService(container, agenda, key, target, options) {
        const done = (job, done) => __awaiter(this, void 0, void 0, function* () {
            try {
                yield container.get(target).execute(job);
                done();
            }
            catch (err) {
                done(err);
                this.errorHandlers.forEach(handler => handler(err));
            }
        });
        if (options) {
            agenda.define(key, options, done);
        }
        else {
            agenda.define(key, done);
        }
    }
}
exports.InversifyAgenda = InversifyAgenda;
class InversifyAgendaService {
    constructor(agenda) {
        this.agenda = agenda;
    }
    now(task, data) {
        try {
            let key;
            if (typeof task === 'string') {
                key = task;
            }
            else {
                const t = exports.inversifyAgendaTasksConfiguration.tasks.find(t => t.target === task);
                key = t.key;
            }
            return this.agenda.now(key, data);
        }
        catch (err) {
            throw new Error('Class is not register as a agenda task');
        }
    }
}
exports.InversifyAgendaService = InversifyAgendaService;
